@@ -109,7 +109,9 @@ static int prop_read_only(enum prop_object_type type,
 				do_clear_received_uuid = true;
 			} else {
 				error(
-"cannot flip ro->rw with received_uuid set, use force if you really want that");
+"cannot flip ro->rw with received_uuid set, use force option -f if you really want unset the read-only status."
+" The value of received_uuid is used for incremental send, consider making a snapshot instead."
+" Read more at btrfs-subvolume(8) and Subvolume flags.");
 				return -EPERM;
 			}
 		}
@@ -175,15 +177,13 @@ static int prop_compression(enum prop_object_type type,
 	int ret;
 	ssize_t sret;
 	int fd = -1;
-	DIR *dirstream = NULL;
 	char *buf = NULL;
 	char *xattr_name = NULL;
 	int open_flags = value ? O_RDWR : O_RDONLY;
 
-	fd = open_file_or_dir3(object, &dirstream, open_flags);
-	if (fd == -1) {
-		ret = -errno;
-		error("failed to open %s: %m", object);
+	fd = btrfs_open_path(object, open_flags == O_RDWR, false);
+	if (fd < 0) {
+		ret = fd;
 		goto out;
 	}
 
@@ -232,7 +232,7 @@ out:
 	free(xattr_name);
 	free(buf);
 	if (fd >= 0)
-		close_file_or_dir(fd, dirstream);
+		close(fd);
 
 	return ret;
 }
